/*
Gwenview - A simple image viewer for KDE
Copyright (C) 2000-2001 Aurlien Gteau

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include <iostream>

#include <qpixmap.h>
#include <qpixmapcache.h>
#include <qprogressdialog.h>

#include <kconfig.h>
#include <klocale.h>

#include "filethumbnailview.moc"


static const char* CONFIG_THUMBNAIL_SIZE="thumbnail size";
static const char* CONFIG_MARGIN_SIZE="margin size";



FileThumbnailView::FileThumbnailView(QWidget* parent) :
	QIconView(parent), KFileView()
{
	setAutoArrange(true);
	setItemsMovable(false);
	setResizeMode(Adjust);

	mProgressDialog=new QProgressDialog(i18n("Generating thumbnails"),i18n("Cancel"),0,this,"progress",true);
	mProgressDialog->setMinimumDuration(0);
}


void FileThumbnailView::readConfig(KConfig* config,QString group) {
	config->setGroup(group);
	mThumbnailSize=config->readNumEntry(CONFIG_THUMBNAIL_SIZE,60);
	mMarginSize=config->readNumEntry(CONFIG_MARGIN_SIZE,10);
	setGridX(mThumbnailSize + mMarginSize*2);
}

void FileThumbnailView::writeConfig(KConfig* config,QString group) const {
	config->setGroup(group);
	config->writeEntry(CONFIG_THUMBNAIL_SIZE,mThumbnailSize);
	config->writeEntry(CONFIG_MARGIN_SIZE,mMarginSize);
}


void FileThumbnailView::insertItem(KFileViewItem* item) {
	if (!item) return;

	QPixmap thumbnail(mThumbnailSize,mThumbnailSize);
	QPainter painter(&thumbnail);
	QPixmap original;
	QString path=item->url().path();
	int xpos,ypos,width,height;

// Init progress dialog if necessary
	if (mProgressDialog->progress()==-1) {
		mProgressDialog->setTotalSteps(numFiles());
		mProgressDialog->setProgress(0);
	}

	if (mProgressDialog->wasCancelled()) {
		painter.eraseRect(0,0,mThumbnailSize,mThumbnailSize);
		painter.drawRect(0,0,mThumbnailSize,mThumbnailSize);
	} else {
	// Load image
		if ( !QPixmapCache::find(path,original) ) {
			original.load(path);
			QPixmapCache::insert(path,original);
		}

		width=original.width();
		height=original.height();

		painter.eraseRect(0,0,mThumbnailSize,mThumbnailSize);
	// Resize needed ?
		if (width>mThumbnailSize || height>mThumbnailSize) {
		// Yes, set painter scale ratio
			double scaleRatio;
			if (width<height) {
				scaleRatio=double(mThumbnailSize)/height;
				xpos=(height-width)/2;
				ypos=0;
			} else {
				scaleRatio=double(mThumbnailSize)/width;
				xpos=0;
				ypos=(width-height)/2;
			}
			painter.scale(scaleRatio,scaleRatio);
		} else {
		// No resize needed
			xpos=(mThumbnailSize-width)/2;
			ypos=(mThumbnailSize-height)/2;
		}

	// Draw thumbnail
		painter.drawPixmap(xpos,ypos,original);

	// Update progress dialog
		mProgressDialog->setProgress(mProgressDialog->progress()+1);
	}
	painter.end();

// Create item
	QIconViewItem* iconItem=new QIconViewItem(this,item->text(),thumbnail);
	item->setViewItem(this,iconItem);
}


void FileThumbnailView::ensureItemVisible(const KFileViewItem* fileItem) {
	if (!fileItem) return;

	QIconViewItem* iconItem=(QIconViewItem*)( fileItem->viewItem(this) );
	QIconView::ensureItemVisible(iconItem);
}


void FileThumbnailView::setSelected(const KFileViewItem* fileItem,bool enable) {
	if (!fileItem) return;

	QIconViewItem* iconItem=(QIconViewItem*)( fileItem->viewItem(this) );
	QIconView::setSelected(iconItem,enable);
}


bool FileThumbnailView::isSelected(const KFileViewItem* fileItem) const {
	if (!fileItem) return false;

	QIconViewItem* current=currentItem();
	if (!current) {
		return false;
	}
	return current->text()==fileItem->text();
}


void FileThumbnailView::removeItem(const KFileViewItem* fileItem) {
	if (!fileItem) return;

	QIconViewItem* iconItem=(QIconViewItem*)( fileItem->viewItem(this) );
	delete iconItem;
	KFileView::removeItem(fileItem);
	arrangeItemsInGrid();
}


void FileThumbnailView::clearView() {
	for (KFileViewItem* item = KFileView::firstItem();item; item=item->next()) {
		item->setViewItem(this,NULL);
	}

// Reset progress dialog here so that next thumbnail generation (which should occur
// right after clearView) will start from 0% even if previous was aborted
	mProgressDialog->reset();
	QIconView::clear();
}
