/*
Gwenview - A simple image viewer for KDE
Copyright (C) 2000-2001 Aurlien Gteau

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include <iostream>

// QT includes
#include <qdir.h>
#include <qheader.h>

// KDE includes
#include <kiconloader.h>
#include <klocale.h>

// Our includes
#include "folderwidget.moc"


/**
 * Create the root node and connect the executed signal to
 * our onExecuted method
 */
FolderWidget::FolderWidget(QWidget* parent) : KListView(parent) {
	FolderWidgetItem* item;
	
	connect(this,SIGNAL(executed(QListViewItem*)),this,SLOT(onExecuted(QListViewItem*)));

	addColumn(i18n("Folders"));

	item=new FolderWidgetItem(this,"/","/");
	
	header()->hide();
}


/**
 * Emit our folderChanged signal
 */
void FolderWidget::onExecuted(QListViewItem* paramItem) {
	FolderWidgetItem* item=(FolderWidgetItem*)(paramItem);

	cout << "Folder changed to : '" << item->folder() << "'\n";
	mFolder=item->folder();
	emit folderChanged(mFolder);
}


/**
 * Open the folder tree to the specified folder and select it
 */
void FolderWidget::setFolder(QString newFolder) {
	QStringList folderParts;
	QStringList::Iterator folderIter,endFolderIter;
	FolderWidgetItem* treeItem;
	FolderWidgetItem* nextTreeItem;

// Init folder
	if (mFolder==newFolder) {
		return;
	}
	
	mFolder=newFolder;
	
// Explore the tree to the specified folder
	folderParts=QStringList::split('/',mFolder);
	endFolderIter=folderParts.end();

	treeItem=(FolderWidgetItem*)(firstChild());
	
	for(folderIter=folderParts.begin();folderIter!=endFolderIter;folderIter++) {
		treeItem->explore();
		nextTreeItem=treeItem->find(*folderIter);
		if (nextTreeItem==NULL) {
			cout << "Can't find folder '" << *folderIter << "'\n";
			break;
		}
		treeItem=nextTreeItem;
	}

// Select the folder
	setSelected(treeItem,true);
	ensureItemVisible(treeItem);
}


/**
 * Find a sub item according to its label
 * ! Assumes that the item tree as already been explored !
 */
FolderWidgetItem* FolderWidgetItem::find(QString label) {
	QListViewItem* child=firstChild();
	
	while(child!=NULL) {
		if (child->text(0)==label) {
			return (FolderWidgetItem*)(child);
		}
		child=child->nextSibling();
	}
	
	return NULL;
}


/**
 * Override setOpen so that if the folder is not explored, we explore it
 * before calling the original implementation.
 */
void FolderWidgetItem::setOpen(bool open) {
	if (!mExplored && open) {
		explore();
	}
	QListViewItem::setOpen(open);
}


/**
 * Called after the FolderWidgetItem object has been appended to the tree
 * Look in the folder for at least one folder. If there's one, add a default tree
 * (So that the plus sign appears for this item) and set 'explored' to false.
 * If there's no folder, set 'explored' to true.
 */
void FolderWidgetItem::init(QString newFolder) {
	mFolder=newFolder;

// Set icon
	setPixmap(0,BarIcon("folders_tab"));

// Check the content for a folder
	QDir dir(mFolder,"*", 0, QDir::Dirs);
	if (!dir.isReadable()) {
		return;
	}
	
	for(unsigned int pos=0;pos<dir.count();pos++) {
	// Skip relative folders (. and ..)
		if (dir[pos]=="." || dir[pos]=="..") {
			continue;
		}
		
	// Mark expandable
		setExpandable(true);
		mExplored=false;
		return;
	}
	
	mExplored=true;
	
}


/**
 * Destroy the existing subtree if any, then create a new one with the last content
 */
void FolderWidgetItem::explore(void) {
	QString entry,subFolder;

	mExplored=true;

// Empty sub tree
	while (firstChild()!=NULL) {
		delete firstChild();
	}

// Get a sorted list of all sub folders
	QDir dir(mFolder,"*",QDir::Name|QDir::IgnoreCase, QDir::Dirs);
	
// Add sub folders to tree
	for(unsigned int pos=0;pos<dir.count();pos++) {
		entry=dir[pos];
		if (entry!="." && entry!="..") {
			subFolder='/' + entry;
			if (mFolder!="/") {
				subFolder=mFolder + subFolder;
			}
			new FolderWidgetItem(this,entry,subFolder);
		}
	}	
}
