/*
Gwenview - A simple image viewer for KDE
Copyright (C) 2000-2001 Aurlien Gteau

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include <iostream>

// QT inludes
#include <qdir.h>
#include <qfile.h>
#include <qpixmapcache.h>

// KDE includes
#include <kaccel.h>
#include <kaction.h>
#include <kapp.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kfiledetailview.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <khelpmenu.h>
#include <kiconloader.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kstdaccel.h>
#include <kstdaction.h>

// Our includes
#include "filewidget.h"
#include "folderwidget.h"
#include "imagefilereader.h"
#include "pixmapwidget.h"

#include "mainwindow.moc"

static const char* CONFIG_DOCK_GROUP="dock";
static const char* CONFIG_MAINWINDOW_GROUP="main window";
static const char* CONFIG_FILEWIDGET_GROUP="file widget";
static const char* CONFIG_START_WITH_THUMBNAILS="start with thumbnails";
static const char* CONFIG_CACHE_SIZE="cache size";


MainWindow::MainWindow() : KDockMainWindow()
{
	bool startWithThumbnails;
	KConfig* config=KGlobal::config();
	config->setGroup(CONFIG_MAINWINDOW_GROUP);
	startWithThumbnails=config->readBoolEntry(CONFIG_START_WITH_THUMBNAILS,false);

	QPixmapCache::setCacheLimit(config->readNumEntry(CONFIG_CACHE_SIZE,4096));

	mFileReader=new ImageFileReader();

// GUI
	createWidgets(startWithThumbnails);
	createActions(startWithThumbnails);
	createMenu();
	
// Connections
	connect(mFolderWidget,SIGNAL(folderChanged(QString)),this,SLOT(setFolder(QString)) );
	connect(mFileWidget,SIGNAL(filenameChanged(QString)),this,SLOT(setFilename(QString)) );

	connect(mFileReader,SIGNAL(newItems(const KFileItemList&)),mFileWidget,SLOT(newItems(const KFileItemList&)) );
	connect(mFileReader,SIGNAL(clear()),mFileWidget,SLOT(clear()) );
	connect(mFileReader,SIGNAL(deleteItem(KFileItem*)),mFileWidget,SLOT(removeItem(KFileItem*)) );
	connect(mFileReader,SIGNAL(completed()),this,SLOT(fileReaderCompleted()) );

// Command line
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	if (args->count()==0) {
		setFolder(QDir::currentDirPath());
	} else {
		QString fullPath=QFile::decodeName(args->arg(0));
		setFullPath(fullPath);
	}
}


MainWindow::~MainWindow() {
    KConfig* config=KGlobal::config();
	delete mFileReader;

// Save main window config
	config->setGroup(CONFIG_MAINWINDOW_GROUP);
	config->writeEntry(CONFIG_START_WITH_THUMBNAILS,mToggleThumbnails->isChecked());
	config->writeEntry(CONFIG_CACHE_SIZE,QPixmapCache::cacheLimit());

// Save widget config
	mFileWidget->writeConfig(config,CONFIG_FILEWIDGET_GROUP);
    writeDockConfig(config,CONFIG_DOCK_GROUP);
}


//-Public slots----------------------------------------------------------
void MainWindow::setFullPath(QString fullPath) {
	QFileInfo fileInfo(fullPath);

// Init mFolder and mFilename
	if (fileInfo.isFile()) {
		mFolder=fileInfo.dirPath(true);
		mFilename=fileInfo.fileName();
		mPixmapWidget->DisplayFile(fullPath);
	} else {
		mFolder=fileInfo.absFilePath();
		mFilename="";
	}

// Update widgets
	mFolderWidget->setFolder(mFolder);
	mFileReader->openURL(KURL(mFolder),true);
}


void MainWindow::setFolder(QString folder){
	if (mFolder==folder) {
		return;
	}
	
	mFolder=folder;
	mFilename="";
	mPixmapWidget->DisplayNothing();

	mFolderWidget->setFolder(mFolder);
	mFileReader->openURL(KURL(mFolder),true);
}


void MainWindow::setFilename(QString filename) {
	if (mFilename==filename) {
		return;
	}

	mFilename=filename;

	if (mFilename!="") {
		mPixmapWidget->DisplayFile(mFolder + '/' + mFilename);
	} else {
		mPixmapWidget->DisplayNothing();
	}
}


//-Private slots---------------------------------------------------------
void MainWindow::deleteFile() {
// Ask confirmation
	if ( KMessageBox::questionYesNo(this,
			i18n("Are you sure you want to delete file '%1' ?").arg(mFilename)
		)==KMessageBox::No)
		return;

// Delete the file
	QString fullPath=mFolder + '/' + mFilename;		
	cout << "Deleting " << fullPath;
	if(QFile::remove(fullPath)) {
		cout << " - done\n";
	} else {
		cout << " - error\n";
	}
}


void MainWindow::renameFile() {
	QString newFilename;
	bool ok;

// Prompt for the new filename
	newFilename=KLineEditDlg::getText(
		i18n("Rename file '%1' to :").arg(mFilename),
		mFilename,
		&ok,this);
		
	if (!ok) return;

// Check that the new file does not already exist
	cout << "Renaming " << mFilename << " to " << newFilename;
	if (QFileInfo(mFolder + '/' + newFilename).exists()) {
		cout << " - error : " << newFilename << " already exists\n";
		return;
	}

// Rename the file
	if(QDir(mFolder).rename(mFilename,newFilename)) {
		cout << " - done\n";
		mFilename=newFilename;
	} else {
		cout << " - error\n";
	}
}


void MainWindow::copyFile() {
	static char block[4096];
	int readLen;

// Prompt for the new folder
    QString newFolder=KFileDialog::getExistingDirectory("/", this);
    if (newFolder=="") return;

// Copy the file
    QFile inputFile(mFolder + '/' + mFilename);
    inputFile.open(IO_Raw | IO_ReadOnly);

    QFile outputFile(newFolder + '/' + mFilename);
    outputFile.open(IO_Raw | IO_WriteOnly);

	while (!inputFile.atEnd()) {
    	readLen=inputFile.readBlock(block,sizeof block);
		outputFile.writeBlock(block,readLen);
	}
}


void MainWindow::toggleThumbnails() {
	if (mToggleThumbnails->isChecked()) {
		mFileWidget->setMode(FileWidget::Thumbnail);
	} else {
		mFileWidget->setMode(FileWidget::FileList);
	}
	mFileReader->listDirectory();
}


void MainWindow::fileReaderCompleted() {
	if (mFilename=="") {
		mFileWidget->selectFirst();
	} else {
		mFileWidget->select(mFilename);
	}
}

//-Internals-------------------------------------------------------------
void MainWindow::createWidgets(bool startWithThumbnails) {
	KConfig* config=KGlobal::config();

	mPixmapDock = createDockWidget("Image",BarIcon("image_tab"),NULL,i18n("Image"));
	mPixmapWidget=new PixmapWidget(mPixmapDock);
	mPixmapDock->setWidget(mPixmapWidget);
	setView(mPixmapDock);
	setMainDockWidget( mPixmapDock);

	mFolderDock = createDockWidget("Folders",BarIcon("folders_tab"),NULL,i18n("Folders"));
	mFolderWidget=new FolderWidget(mFolderDock);
	mFolderDock->setWidget(mFolderWidget);

	mFileDock = createDockWidget("Files",BarIcon("files_tab"),NULL,i18n("Files"));
	mFileWidget=new FileWidget(this,startWithThumbnails?FileWidget::Thumbnail:FileWidget::FileList);
	mFileWidget->readConfig(config,CONFIG_FILEWIDGET_GROUP);
	mFileDock->setWidget(mFileWidget);

// Default dock config
    this->setGeometry(100,100,500,400);
    mFolderDock->manualDock( mPixmapDock,KDockWidget::DockLeft,20);
    mFileDock->manualDock( mPixmapDock,KDockWidget::DockTop,20);

// Load dock config
    readDockConfig(config,CONFIG_DOCK_GROUP);
}



void MainWindow::createActions(bool startWithThumbnails) {
	mRenameFile=new KAction(i18n("&Rename"),KAccel::stringToKey("F2"),this,SLOT(renameFile()),this);
	mCopyFile=new KAction(i18n("&Copy"),KAccel::stringToKey("F5"),this,SLOT(copyFile()),this);
	mDeleteFile=new KAction(i18n("&Delete"),KAccel::stringToKey("Delete"),this,SLOT(deleteFile()),this);

	mToggleThumbnails=new KToggleAction(i18n("Thumbnails"),KAccel::stringToKey("T"),this,SLOT(toggleThumbnails()),this);
	mToggleThumbnails->setChecked(startWithThumbnails);
	
	mSelectFirst=new KAction(i18n("&First"),KAccel::stringToKey("Home"),mFileWidget,SLOT(selectFirst()),this);
	mSelectPrevious=KStdAction::back(mFileWidget, SLOT(selectPrevious()) );
	mSelectPrevious->setAccel(KAccel::stringToKey("Backspace"));
	mSelectNext=KStdAction::forward(mFileWidget, SLOT(selectNext()) );
	mSelectNext->setAccel(KAccel::stringToKey("Space"));
	mSelectLast=new KAction(i18n("&Last"),KAccel::stringToKey("End"),mFileWidget,SLOT(selectLast()),this);
}



void MainWindow::createMenu() {
	QPopupMenu* fileMenu = new QPopupMenu;

	mRenameFile->plug(fileMenu);
	mCopyFile->plug(fileMenu);
	mDeleteFile->plug(fileMenu);
	KStdAction::quit( kapp, SLOT (closeAllWindows()) )->plug (fileMenu);
	menuBar()->insertItem(i18n("&File"), fileMenu);

	QPopupMenu* viewMenu = new QPopupMenu;
	mSelectFirst->plug(viewMenu);
	mSelectPrevious->plug(viewMenu);
	mSelectNext->plug(viewMenu);
	mSelectLast->plug(viewMenu);
	viewMenu->insertSeparator();
	mToggleThumbnails->plug(viewMenu);
	menuBar()->insertItem(i18n("&View"), viewMenu);

	QPopupMenu* windowMenu = dockHideShowMenu();
	menuBar()->insertItem(i18n("&Window"), windowMenu);

	menuBar()->insertItem(i18n("&Help"), helpMenu());
}
